
function [betaNew, conv] = betaGivenOmega(X, Y, mask, thWay, errBnd, maxIT, betaInit, omegaEst, trnScales, lambda, output )

% This is the Beta-optimization step of FLOG

[n, d] = size(X); 
k0 = norm(X, 2); X = X ./ k0; betaInit = betaInit./k0; 

%%%%%%%%%%% Initionaliztion %%%%%%%%%%%%%%%%%%%%%%%%%%
betaCur =  betaInit; betaNew =  betaInit;
j = 0;
XtX = X'*X;
XtY = X'*Y;
fcur = objectiveValue( X, Y, betaInit, omegaEst, thWay, lambda);

%%%%%%%%%%% Iteration starts %%%%%%%%%%%%%%%%%%%%%%%%%
alpha = 1;
while 1
    j = j + 1;
    
    G = (XtX*betaCur-XtY)*omegaEst;
    penG = lambda*sign(betaCur);
    
    %% armijo search
    % alpha = 1;
    while (alpha>1e-6)
        xi_new = betaCur - alpha*G;
        
        %% screening by mask
        xi_new(mask==0) = 0;
        
        %% thresholding
        betaNew = betaThresholding( xi_new, thWay, lambda);
        
        %% evaluate
        fnew = objectiveValue( X, Y, betaNew, omegaEst, thWay, lambda);
        gap = 1e-4*trace((betaNew-betaCur)'*(G+penG));  
        if fnew<=fcur+gap
            break;
        else
            alpha = alpha/10;
        end
    end
    
    %% check convergence
    err = sum(sum(abs(betaCur-betaNew)));  
    if  err <= errBnd 
        conv = 1; break;
    end
    
    if j>=maxIT
        if output, disp(['no converge. err = ', num2str(err)]); end
        conv = 0; break;
    end
    
    %%% update the estimate
    betaCur = betaNew;
    fcur = fnew;
end

%%%%%%%%%%%%%%%%%% iteration ends %%%%%%%%%%%%%%%%%%%%%%%
if (any(any(isnan(betaNew)))) || (any(any(isinf(betaNew))))
    disp('inregular elements!');
end

betaNew = betaNew/k0;

if strcmp(thWay, 'quantile')
    betaNew = betaNew./repmat(trnScales', 1, d); % trnScaleMat = repmat(trnScales', 1, d);
end

end


function [ betaNew ] = betaThresholding( Beta, thWay, lambda)

d = size(Beta, 1);

switch(lower(thWay))
    case 'quantile'
        Beta0 = Beta - diag(diag(Beta));
        sortNorm = sort(abs(Beta0(:)), 'descend');
        th = sortNorm(lambda);
        ttInds = abs(Beta0)>th;   
        betaNew = diag(diag(Beta))/(1+1e-8);
        betaNew(ttInds) =  Beta0(ttInds)/(1+1e-8) ;

    case 'hard'
        ttInds = abs(Beta)>lambda;
        betaNew = zeros(d);
        betaNew(ttInds) =  Beta(ttInds);

    case 'soft'
        absV = abs(Beta);
        ttInds = absV>lambda;
        betaNew = zeros(d);          
        betaNew(ttInds) = sign(Beta(ttInds)).*(absV(ttInds)-lambda);
end
        
end


function [ value ] = objectiveValue( X, Y, Beta, Omega, thWay, lambda)

traceTerm = trace((X*Beta-Y)*Omega*(X*Beta-Y)')/2;

if strcmp(thWay,'soft')
    penalty = sum(sum(lambda.* abs(Beta)));
elseif strcmp(thWay,'hard')
    penalty = sum(sum(lambda.* (Beta~=0)));
else
    penalty =0 ;
end
value = traceTerm  + penalty;

end