function [X, trnCenters, trnScales, Y, trnCenterY, valX, tstX, valY, tstY] = dataStand(centerX, scaleX, X, centerY, Y, valX, tstX, valY, tstY)
% Use the parameters obtained from the training data to standardize the
% data (including the validation and the test data) for both the design and
% the response.
trnCenters = []; trnScales = []; trnCenterY = [];
if centerX == 1 % X should not include the intercept column
    % Note y-centering is not appropriate for nonGaussian GLM (change the distr)
    trnCenters = mean(X);
    X = X - repmat(trnCenters, size(X, 1), 1);
    if ~isempty(valX)
        valX = valX - repmat(trnCenters, size(valX, 1), 1);
    else
        valX = [];
    end
    if ~isempty(tstX)
        tstX = tstX - repmat(trnCenters, size(tstX, 1), 1);
    else
        tstX = [];
    end
end
if scaleX == 1 % Assume X does not include the intercept column in this case
    % Col-normalize X to have all L2-norms equal to sqrt(n)/sqrt(valN)/sqrt(tstN)
    % Use the same params to scale val & tst
    trnScales = sqrt(sum(X.^2)/size(X, 1));
    X = X .* repmat(1./trnScales, size(X, 1), 1); % X * diag(1./(trnScales));
    if ~isempty(valX)
        valX = valX .* repmat(1./trnScales, size(valX, 1), 1);
    end
    %     if ~isempty(tstX)
    %         tstX = tstX .* repmat(1./trnScales, size(tstX, 1), 1);
    %     end
else
    trnScales = ones(size(mean(X)));
end
if centerY == 1
    trnCenterY = mean(Y);
    Y = Y - repmat(trnCenterY, size(Y, 1), 1);
    %Y = Y - trnCenterY;
    if ~isempty(valY)
        valY = valY - repmat(trnCenterY, size(valY, 1), 1);
    else
        valY = [];
    end
    if ~isempty(tstY)
        tstY = tstY - repmat(trnCenterY, size(tstY, 1), 1);
    else
        tstY = [];
    end
end
end



