function [RI, pSpa, pNz] = evaluateCluster(clustStat, trueClustStat )

% This function evaluates the accuracy of the clustering result
% evaluate the decomposition results using Rand index (RI):
% RI=?TP+TF)/(TP+TF+FP+FN)
% TP: a pair coming from the same cluster are assigned to a single cluster
% TF: a pair coming from different clusters are assigned to different clusters
% FP: a pair coming from different clusters are assigned to the same cluster
% FN: a pair coming from the same cluster are assigned to different clusters

clustMat = clusterMat(clustStat);
trueClustMat = clusterMat(trueClustStat);
pSpa = nnz(clustMat&trueClustMat)/nnz(trueClustMat);
pNz = nnz((~clustMat)&(~trueClustMat))/nnz(~trueClustMat);

TP = nnz(clustMat&trueClustMat); 
TN = nnz((~clustMat)&(~trueClustMat));
FP = nnz(clustMat&(~trueClustMat));
PN = nnz(~clustMat&trueClustMat); 

RI = (TP+TN) / (TP+TN+FP+PN);
end

function [ clusterMat ] = clusterMat( clusterLabel )

d = length(clusterLabel);
clusterMat = zeros(d);
for i = 1:d
    for j = 1:d
        if (clusterLabel(i) == clusterLabel(j))
            clusterMat(i,j) = 1;
        else
            clusterMat(i,j) = 0;
        end
    end
end

end
