function [ Beta, Sigma, Omega, Joint, labels ] = generateModel( d, ds, rho, phi)

% This function simulates B and Omega for the network model
% d: a scalar. network size.
% ds: a vector. subnetwork sizes.

subN = length(ds);

%% Group labels
labels = zeros(d,1);
start = 0;
for i = 1:subN
    d0 = ds(i);
    labels(start+1:start+d0) = i;
    start = start + d0;
end

%% Beta
Beta = zeros(d);
start = 0;
for i = 1:subN
    d0 = ds(i);
    Bblock = betaBlock(d0);
    Beta(start+1:start+d0, start+1:start+d0) = Bblock;
    start = start + d0;
end

%% Sigma
[Sigma, Omega] = sigmaOmega( d, ds, rho );

%% Joint
Joint = sqrt( Beta.^2+Beta'.^2 + phi*( Omega.^2+ Omega'.^2) );

end

function [ Beta ] = betaBlock( d )
geInd = 0;
while 1    
    % This method is used by MRCE
    s1 = 0.1;
    s2 = 1;
    W = randn(d);
    K = binornd(1,s1,d,d);
    q = binornd(1,s2,d,1);
    Q = repmat(q,1,d);
    Beta = W.*K.*Q;
    
    % for stationarity
    if max(abs(eig(Beta)))>=1
        Beta = Beta * 0.9 /max(abs(eig(Beta)));
    end
    
    % check if every node is connected
    tempMat = Beta-diag(diag(Beta));
    if all(sum(tempMat~=0)+sum(tempMat'~=0)~=0)
        break;
    end
    
    geInd = geInd+1;
    if geInd>=10000
        error('difficult to generate required network! may need to change parameters!')
    end
end

end

function [Sigma, Omega] = sigmaOmega( d, ds, rho )

subN = length(ds);

Omega = zeros(d);
start = 0;
for i = 1:subN
    d0 = ds(i);
    Oblock = randPD( d0 , rho);
    Omega(start+1:start+d0, start+1:start+d0) = Oblock;
    start = start + d0;
end
Sigma = inv(Omega);

end

function [ M ] = randPD( d, rho )

D = randn(d);
if ~isempty(rho)
    s = sort(abs(D(:)),'descend');
    th = s(floor(rho*d*d));
    D(abs(D)<th) = 0;
end
M = D+D';
M = M/max(abs(M(:)));
M = M-diag(diag(M))+eye(d);
while ~ispd(M)
    M = M+0.1*eye(d);
end

M = M/M(1,1);

end